@@ -30,19 +30,6 @@
 
 #define DEBUG_VARIABLE
 
-/*
- * Types are private:
- */
-
-typedef struct _xsltStack xsltStack;
-typedef xsltStack *xsltStackPtr;
-struct _xsltStack {
-    int cur;
-    int max;
-    xsltStackElemPtr elems[50];
-};
-
-
 /************************************************************************
  *									*
  *			Module interfaces				*
@@ -50,7 +37,7 @@ struct _xsltStack {
  ************************************************************************/
 
 /**
- * xsltNewParserContext:
+ * xsltNewStackElem:
  *
  * Create a new XSLT ParserContext
  *
@@ -111,6 +98,38 @@ xsltFreeStackElemList(xsltStackElemPtr elem) {
     }
 }
 
+/**
+ * xsltCheckStackElem:
+ * @ctxt:  xn XSLT transformation context
+ * @name:  the variable name
+ * @nameURI:  the variable namespace URI
+ *
+ * check wether the variable or param is already defined
+ *
+ * Returns 1 if present, 0 if not, -1 in case of failure.
+ */
+int
+xsltCheckStackElem(xsltTransformContextPtr ctxt, const xmlChar *name,
+	           const xmlChar *nameURI) {
+    xsltStackElemPtr cur;
+
+    if ((ctxt == NULL) || (name == NULL))
+	return(-1);
+
+    cur = ctxt->vars;
+    while (cur != NULL) {
+	if (xmlStrEqual(name, cur->name)) {
+	    if (((nameURI == NULL) && (cur->nameURI == NULL)) ||
+		((nameURI != NULL) && (cur->nameURI != NULL) &&
+		 (xmlStrEqual(nameURI, cur->nameURI)))) {
+		return(1);
+	    }
+	}
+	cur = cur->next;
+    }
+    return(0);
+}
+
 /**
  * xsltAddStackElem:
  * @ctxt:  xn XSLT transformation context
@@ -122,104 +141,43 @@ xsltFreeStackElemList(xsltStackElemPtr elem) {
  */
 int
 xsltAddStackElem(xsltTransformContextPtr ctxt, xsltStackElemPtr elem) {
-    xsltStackPtr stack;
-    xsltStackElemPtr cur;
-
     if ((ctxt == NULL) || (elem == NULL))
 	return(-1);
 
-    stack = ctxt->variablesHash;
-    if (stack == NULL) {
-	/* TODO make stack size dynamic !!! */
-	stack = xmlMalloc(sizeof (xsltStack));
-	if (stack == NULL) {
-	    xsltGenericError(xsltGenericErrorContext,
-		"xsltPushStack : malloc failed\n");
-	    return(-1);
-	}
-	memset(stack, 0, sizeof(xsltStack));
-	ctxt->variablesHash = stack;
-	stack->cur = 0;
-	stack->max = 50;
-    }
-    /* TODO: check that there is no conflict with existing values
-     * at that level */
-    cur = stack->elems[stack->cur];
-    while (cur != NULL) {
-	if (xmlStrEqual(elem->name, cur->name)) {
-	    if (((elem->nameURI == NULL) && (cur->nameURI == NULL)) ||
-		((elem->nameURI != NULL) && (cur->nameURI != NULL) &&
-		 (xmlStrEqual(elem->nameURI, cur->nameURI)))) {
-		xsltGenericError(xsltGenericErrorContext,
-		    "redefinition of param or variable %s\n", elem->name);
-		xsltFreeStackElem(elem);
-	    }
-	}
-	cur = cur->next;
-    }
-
-    elem->next = stack->elems[stack->cur];
-    stack->elems[stack->cur] = elem;
+    elem->next = ctxt->varsTab[ctxt->varsNr - 1];
+    ctxt->varsTab[ctxt->varsNr - 1] = elem;
+    ctxt->vars = elem;
     return(0);
 }
 
 /**
- * xsltPushStack:
+ * xsltAddStackElemList:
  * @ctxt:  xn XSLT transformation context
+ * @elems:  a stack element list
  *
- * Push a new level on the ctxtsheet interprestation stack
- */
-void
-xsltPushStack(xsltTransformContextPtr ctxt) {
-    xsltStackPtr stack;
-
-    if (ctxt == NULL)
-	return;
-
-    stack = ctxt->variablesHash;
-    if (stack == NULL) {
-	/* TODO make stack size dynamic !!! */
-	stack = xmlMalloc(sizeof (xsltStack));
-	if (stack == NULL) {
-	    xsltGenericError(xsltGenericErrorContext,
-		"xsltPushStack : malloc failed\n");
-	    return;
-	}
-	memset(stack, 0, sizeof(xsltStack));
-	ctxt->variablesHash = stack;
-	stack->cur = 0;
-	stack->max = 50;
-    }
-    if (stack->cur >= stack->max + 1) {
-	TODO /* make stack size dynamic !!! */
-	xsltGenericError(xsltGenericErrorContext,
-	    "xsltPushStack : overflow\n");
-	return;
-    }
-    stack->cur++;
-    stack->elems[stack->cur] = NULL;
-}
-
-/**
- * xsltPopStack:
- * @ctxt:  an XSLT transformation context
+ * add the new element list at this level of the stack.
  *
- * Pop a level on the ctxtsheet interprestation stack
+ * Returns 0 in case of success, -1 in case of failure.
  */
-void
-xsltPopStack(xsltTransformContextPtr ctxt) {
-    xsltStackPtr stack;
+int
+xsltAddStackElemList(xsltTransformContextPtr ctxt, xsltStackElemPtr elems) {
+    xsltStackElemPtr cur;
 
-    if (ctxt == NULL)
-	return;
+    if ((ctxt == NULL) || (elems == NULL))
+	return(-1);
 
-    stack = ctxt->variablesHash;
-    if (stack == NULL)
-	return;
-
-    xsltFreeStackElemList(stack->elems[stack->cur]);
-    stack->elems[stack->cur] = NULL;
-    stack->cur--;
+    /* TODO: check doublons */
+    if (ctxt->varsTab[ctxt->varsNr - 1] != NULL) {
+	cur = ctxt->varsTab[ctxt->varsNr - 1];
+	while (cur->next != NULL)
+	    cur = cur->next;
+	cur->next = elems;
+    } else {
+	elems->next = ctxt->varsTab[ctxt->varsNr - 1];
+	ctxt->varsTab[ctxt->varsNr - 1] = elems;
+	ctxt->vars = elems;
+    }
+    return(0);
 }
 
 /**
@@ -234,36 +192,30 @@ xsltStackElemPtr
 xsltStackLookup(xsltTransformContextPtr ctxt, const xmlChar *name,
 	        const xmlChar *nameURI) {
     xsltStackElemPtr ret = NULL;
-    xsltStackPtr stack;
     int i;
     xsltStackElemPtr cur;
 
     if ((ctxt == NULL) || (name == NULL))
 	return(NULL);
 
-    stack = ctxt->variablesHash;
-    if (stack == NULL)
-	return(NULL);
+    /*
+     * Do the lookup from the top of the stack, but
+     * don't use params being computed in a call-param
+     */
+    i = ctxt->varsNr - 1;
 
-    for (i = stack->cur;i >= 0;i--) {
-	cur = stack->elems[i];
+    for (;i >= 0;i--) {
+	cur = ctxt->varsTab[i];
 	while (cur != NULL) {
 	    if (xmlStrEqual(cur->name, name)) {
-		/* TODO: double check param binding */
 		if (nameURI == NULL) {
 		    if (cur->nameURI == NULL) {
-			if (cur->type == XSLT_ELEM_PARAM)
-			    ret = cur;
-			else
-			    return(cur);
+			return(cur);
 		    }
 		} else {
 		    if ((cur->nameURI != NULL) &&
 			(xmlStrEqual(cur->nameURI, nameURI))) {
-			if (cur->type == XSLT_ELEM_PARAM)
-			    ret = cur;
-			else
-			    return(cur);
+			return(cur);
 		    }
 		}
 
@@ -333,12 +285,11 @@ xsltEvalVariables(xsltTransformContextPtr ctxt, xsltStackElemPtr elem) {
 	    if (elem->value != NULL)
 		xmlXPathFreeObject(elem->value);
 	    elem->value = result;
-	    elem->computed = 1;
 	}
+	elem->computed = 1;
     } else {
 	if (elem->tree == NULL) {
 	    elem->value = xmlXPathNewCString("");
-	    elem->computed = 1;
 	} else {
 	    /*
 	     * This is a result tree fragment.
@@ -350,17 +301,28 @@ xsltEvalVariables(xsltTransformContextPtr ctxt, xsltStackElemPtr elem) {
 		                      (const xmlChar *) "fake", NULL);
 	    if (container == NULL)
 		return(-1);
+
 	    oldInsert = ctxt->insert;
 	    oldNode = ctxt->node;
 	    ctxt->insert = container;
-
-	    xsltApplyOneTemplate(ctxt, ctxt->node, NULL, elem->tree);
-
+	    xsltApplyOneTemplate(ctxt, ctxt->node, elem->tree, 0);
 	    ctxt->insert = oldInsert;
 	    ctxt->node = oldNode;
+
+	    if (elem->value != NULL)
+		xmlXPathFreeObject(elem->value);
 	    elem->value = xmlXPathNewValueTree(container);
-	    elem->computed = 1;
+	    if (elem->value == NULL) {
+		elem->value = xmlXPathNewCString("");
+	    }
+#ifdef DEBUG_VARIABLE
+	    if ((xsltGenericDebugContext == stdout) ||
+		(xsltGenericDebugContext == stderr))
+		xmlXPathDebugDumpObject((FILE *)xsltGenericDebugContext,
+					elem->value, 0);
+#endif
 	}
+	elem->computed = 1;
     }
     return(0);
 }
@@ -387,14 +349,15 @@ xsltEvalGlobalVariables(xsltTransformContextPtr ctxt) {
 #endif
     ctxt->node = (xmlNodePtr) ctxt->document->doc;
     style = ctxt->style;
-    /* TODO: handle the stylesheet cascade */
-    if (style != NULL) {
+    while (style != NULL) {
 	elem = style->variables;
 	
 	while (elem != NULL) {
 	    xsltEvalVariables(ctxt, elem);
 	    elem = elem->next;
 	}
+
+	style = xsltNextImport(style);
     }
     return(0);
 }
@@ -448,6 +411,56 @@ xsltRegisterGlobalVariable(xsltStylesheetPtr style, const xmlChar *name,
     return(0);
 }
 
+/**
+ * xsltBuildVariable:
+ * @ctxt:  the XSLT transformation context
+ * @name:  the variable name
+ * @ns_uri:  the variable namespace URI
+ * @select:  the expression which need to be evaluated to generate a value
+ * @tree:  the tree if select is NULL
+ * @param:  this is a parameter actually
+ *
+ * Computes a new variable value.
+ *
+ * Returns the xsltStackElemPtr or NULL in case of error
+ */
+xsltStackElemPtr
+xsltBuildVariable(xsltTransformContextPtr ctxt, const xmlChar *name,
+		     const xmlChar *ns_uri, const xmlChar *select,
+		     xmlNodePtr tree, int param) {
+    xsltStackElemPtr elem;
+    if (ctxt == NULL)
+	return(NULL);
+    if (name == NULL)
+	return(NULL);
+
+#ifdef DEBUG_VARIABLE
+    xsltGenericDebug(xsltGenericDebugContext,
+		     "Building variable %s", name);
+    if (select != NULL)
+	xsltGenericDebug(xsltGenericDebugContext,
+			 " select %s",  select);
+    xsltGenericDebug(xsltGenericDebugContext, "\n");
+#endif
+    elem = xsltNewStackElem();
+    if (elem == NULL)
+	return(NULL);
+    if (param)
+	elem->type = XSLT_ELEM_PARAM;
+    else
+	elem->type = XSLT_ELEM_VARIABLE;
+    elem->name = xmlStrdup(name);
+    if (select != NULL)
+	elem->select = xmlStrdup(select);
+    else
+	elem->select = NULL;
+    if (ns_uri)
+	elem->nameURI = xmlStrdup(ns_uri);
+    elem->tree = tree;
+    xsltEvalVariables(ctxt, elem);
+    return(elem);
+}
+
 /**
  * xsltRegisterVariable:
  * @ctxt:  the XSLT transformation context
@@ -457,8 +470,7 @@ xsltRegisterGlobalVariable(xsltStylesheetPtr style, const xmlChar *name,
  * @tree:  the tree if select is NULL
  * @param:  this is a parameter actually
  *
- * Register a new variable value. If @value is NULL it unregisters
- * the variable
+ * Computes and register a new variable value. 
  *
  * Returns 0 in case of success, -1 in case of error
  */
@@ -472,31 +484,20 @@ xsltRegisterVariable(xsltTransformContextPtr ctxt, const xmlChar *name,
     if (name == NULL)
 	return(-1);
 
+    if (xsltCheckStackElem(ctxt, name, ns_uri) != 0) {
+	if (!param) {
+	    xsltGenericError(xsltGenericErrorContext,
+	    "xsl:variable : redefining %s\n", name);
+	}
 #ifdef DEBUG_VARIABLE
-    xsltGenericDebug(xsltGenericDebugContext,
-		     "Defineing variable %s", name);
-    if (select != NULL)
-	xsltGenericDebug(xsltGenericDebugContext,
-			 " select %s",  select);
-    xsltGenericDebug(xsltGenericDebugContext, "\n");
+	else
+	    xsltGenericDebug(xsltGenericDebugContext,
+		     "param %s defined by caller", name);
 #endif
-    elem = xsltNewStackElem();
-    if (elem == NULL)
-	return(-1);
-    if (param)
-	elem->type = XSLT_ELEM_PARAM;
-    else
-	elem->type = XSLT_ELEM_VARIABLE;
-    elem->name = xmlStrdup(name);
-    if (select != NULL)
-	elem->select = xmlStrdup(select);
-    else
-	elem->select = NULL;
-    if (ns_uri)
-	elem->nameURI = xmlStrdup(ns_uri);
-    elem->tree = tree;
+	return(0);
+    }
+    elem = xsltBuildVariable(ctxt, name, ns_uri, select, tree, param);
     xsltAddStackElem(ctxt, elem);
-    xsltEvalVariables(ctxt, elem);
     return(0);
 }
 
@@ -523,7 +524,6 @@ xsltGlobalVariableLookup(xsltTransformContextPtr ctxt, const xmlChar *name,
 	
 	while (elem != NULL) {
 	    if (xmlStrEqual(elem->name, name)) {
-		/* TODO: double check param binding */
 		if (ns_uri == NULL) {
 		    if (elem->nameURI == NULL)
 			break;
@@ -597,30 +597,79 @@ xsltVariableLookup(xsltTransformContextPtr ctxt, const xmlChar *name,
     return(NULL);
 }
 
-
 /**
- * xsltFreeVariableHashes:
- * @ctxt: an XSLT transformation context
+ * xsltParseStylesheetCallerParam:
+ * @ctxt:  the XSLT transformation context
+ * @cur:  the "param" element
  *
- * Free up the memory used by xsltAddVariable/xsltGetVariable mechanism
+ * parse an XSLT transformation param declaration, compute
+ * its value but doesn't record it.
+ *
+ * It returns the new xsltStackElemPtr or NULL
  */
-void
-xsltFreeVariableHashes(xsltTransformContextPtr ctxt) {
-    xsltStackPtr stack;
-    int i;
 
-    if (ctxt == NULL)
-	return;
+xsltStackElemPtr
+xsltParseStylesheetCallerParam(xsltTransformContextPtr ctxt, xmlNodePtr cur) {
+    xmlChar *name, *ncname, *prefix;
+    xmlChar *select;
+    xmlNodePtr tree = NULL;
+    xsltStackElemPtr elem;
 
-    stack = ctxt->variablesHash;
-    if (stack == NULL)
-	return;
+    if ((cur == NULL) || (ctxt == NULL))
+	return(NULL);
 
-    for (i = 0; i <= stack->cur;i++) {
-	xsltFreeStackElemList(stack->elems[i]);
+    name = xmlGetNsProp(cur, (const xmlChar *)"name", XSLT_NAMESPACE);
+    if (name == NULL) {
+	xsltGenericError(xsltGenericErrorContext,
+	    "xsl:param : missing name attribute\n");
+	return(NULL);
     }
-    xmlFree(stack);
 
+#ifdef DEBUG_VARIABLE
+    xsltGenericDebug(xsltGenericDebugContext,
+	"Parsing param %s\n", name);
+#endif
+
+    select = xmlGetNsProp(cur, (const xmlChar *)"select", XSLT_NAMESPACE);
+    if (select == NULL) {
+	tree = cur->children;
+    } else {
+#ifdef DEBUG_VARIABLE
+	xsltGenericDebug(xsltGenericDebugContext,
+	    "        select %s\n", select);
+#endif
+	if (cur->children != NULL)
+	    xsltGenericError(xsltGenericErrorContext,
+	    "xsl:param : content shuld be empty since select is present \n");
+    }
+
+    ncname = xmlSplitQName2(name, &prefix);
+
+    if (ncname != NULL) {
+	if (prefix != NULL) {
+	    xmlNsPtr ns;
+
+	    ns = xmlSearchNs(cur->doc, cur, prefix);
+	    if (ns == NULL) {
+		xsltGenericError(xsltGenericErrorContext,
+		    "xsl:param : no namespace bound to prefix %s\n", prefix);
+	    } else {
+		elem = xsltBuildVariable(ctxt, ncname, ns->href, select,
+			                 tree, 1);
+	    }
+	    xmlFree(prefix);
+	} else {
+	    elem = xsltBuildVariable(ctxt, ncname, NULL, select, tree, 1);
+	}
+	xmlFree(ncname);
+    } else {
+	elem = xsltBuildVariable(ctxt, name, NULL, select, tree, 1);
+    }
+
+    xmlFree(name);
+    if (select != NULL)
+	xmlFree(select);
+    return(elem);
 }
 
 /**
@@ -657,6 +706,10 @@ xsltParseStylesheetParam(xsltTransformContextPtr ctxt, xmlNodePtr cur) {
     if (select == NULL) {
 	tree = cur->children;
     } else {
+#ifdef DEBUG_VARIABLE
+	xsltGenericDebug(xsltGenericDebugContext,
+	    "        select %s\n", select);
+#endif
 	if (cur->children != NULL)
 	    xsltGenericError(xsltGenericErrorContext,
 	    "xsl:param : content shuld be empty since select is present \n");
