@@ -151,9 +151,10 @@ struct _xsltStylesheet {
     xmlHashTablePtr attributeSets;/* the attribute sets hash tables */
 
     /*
-     * Attribute sets
+     * Namespaces
      */
     xmlHashTablePtr nsHash;     /* the set of namespaces in use */
+    void           *nsDefs;     /* the namespaces defined */
 
     /*
      * Key definitions
@@ -245,6 +246,7 @@ typedef void (*xsltTransformFunction) (xsltTransformContextPtr ctxt,
 xsltDecimalFormatPtr	xsltDecimalFormatGetByName(xsltStylesheetPtr sheet,
 						   xmlChar *name);
  */
+xsltStylesheetPtr	xsltNewStylesheet	(void);
 xsltStylesheetPtr	xsltParseStylesheetFile	(const xmlChar* filename);
 void			xsltFreeStylesheet	(xsltStylesheetPtr sheet);
 int			xsltIsBlank		(xmlChar *str);
@@ -254,6 +256,8 @@ xsltDecimalFormatPtr	xsltDecimalFormatGetByName(xsltStylesheetPtr sheet,
 
 xsltStylesheetPtr	xsltParseStylesheetProcess(xsltStylesheetPtr ret,
 	                                         xmlDocPtr doc);
+void			xsltParseStylesheetOutput(xsltStylesheetPtr style,
+						  xmlNodePtr cur);
 xsltStylesheetPtr	xsltParseStylesheetDoc	(xmlDocPtr doc);
 #ifdef __cplusplus
 }
